@@ -92,18 +92,21 @@
 #'  sampled time extrapolated to infinity based on the last predicted 
 #'  concentration obtained from the regression line used to estimate Lambda_z 
 #'  (Clast_pred).
-#'  \item \strong{Vss_obs and Vss_pred} are estimated volume of distribution at 
-#'  steady-state based on Clast_obs and Clast_pred, respectively.
 #'  \item \strong{Tau} is the dosing interval for steady-state data. This value 
 #'  is assumed equarion over multiple doses.
 #'  \item \strong{Cmin and Tmin} are the minimum concentration between 0 and 
 #'  Tau and the corresponding time, respectively.
 #'  \item \strong{Cavg} is the average concentration between 0 and Tau for 
 #'  steady-state data.
+#'  \item \strong{AUCtau and AUMCtau} are AUC and AUMC between 0 and Tau for 
+#'  steady-state data.
+#'  \item \strong{Clss} is an estimate of the total body clearance for
+#'  steady-state data.
+#'  \item \strong{Vss_obs and Vss_pred} are estimated volume of distribution at 
+#'  steady-state based on Clast_obs and Clast_pred, respectively.
 #'  \item \strong{p_Fluctuation} is the percentage of the fluctuation of the 
 #'  concentration between 0 and Tau for steady-state data.
 #'  \item \strong{Accumulation_Index} is \eqn{1/(1-e^(-\lambda_z*\tau))}
-#'  \item \strong{Clss} is an estimate of the total body clearance for steady-state data.
 #' }
 #' 
 #' @param time Numeric array for time
@@ -114,22 +117,26 @@
 #' @param doseType Steady-state (ss) or nonsteady-state (ns) dose
 #'   (\strong{"ns"})
 #' @param adminType Route of administration
-#'   (iv-bolus,iv-infusion,ss,extravascular) (\strong{"extravascular"})
+#'   (iv-bolus,iv-infusion,extravascular) (\strong{"extravascular"})
 #' @param doseNm Column name to specify dose identifiers (\strong{"NULL"})
-#' @param dose Dose identifiers used in the entire data (c(1,2)) (\strong{"NULL"})
-#' @param doseNumber Dose identifier used in this estimation (1) (\strong{"NULL"})
+#' @param dose Dose identifiers used in the entire data (c(1,2))
+#'   (\strong{"NULL"})
+#' @param doseNumber Dose identifier used in this estimation (1)
+#'   (\strong{"NULL"})
 #' @param doseAmt Dose amounts (\strong{"NULL"})
 #' @param method linear, loglinear or mixed (\strong{"linear"})
-#' @param AUCTimeRange User-defined window of time used to estimate AUC
+#' @param AUCTimeRange User-defined window of time used to estimate AUC 
 #'   (\strong{"NULL"})
-#' @param LambdaTimeRange User-defined window of time to estimate elimination
+#' @param LambdaTimeRange User-defined window of time to estimate elimination 
 #'   rate-constant (\strong{"NULL"})
-#' @param LambdaExclude User-defined excluded observations during estimation of 
-#'   elimination rate-constant (\strong{"NULL"})
+#' @param LambdaExclude User-defined excluded observation time points for
+#'   estimation of elimination rate-constant (\strong{"NULL"})
 #' @param Tau Dosing interval for steady-state data (\strong{"NULL"})
 #' @param TI Infusion duration (\strong{"NULL"})
-#' @param simFile Name of the simulated concentration-time data if present (\strong{"NULL"})
-#' @param dset Type, i.e., observed or simulated concentration-time data set ("obs" or "sim") (\strong{"obs"})
+#' @param simFile Name of the simulated concentration-time data if present
+#'   (\strong{"NULL"})
+#' @param dset Type, i.e., observed or simulated concentration-time data set
+#'   ("obs" or "sim") (\strong{"obs"})
 #' 
 #' @return An array of estimated NCA metrics
 #' @export
@@ -140,9 +147,20 @@
 # function to estimate NCA parameters
 est.nca <- function(time,conc,backExtrp="FALSE",negConcExcl="FALSE",doseType="ns",adminType="extravascular",doseNm=NULL,dose=NULL,doseNumber=NULL,doseAmt=NULL,method="linear",AUCTimeRange=NULL,LambdaTimeRange=NULL,LambdaExclude=NULL,Tau=NULL,TI=NULL,simFile=NULL,dset="obs"){
   
-  ntime <- time
-  nconc <- conc
-  ndose <- length(dose)
+  "tail" <- "head" <- "lm" <- "coef" <- NULL
+  rm(list=c("tail","head","lm","coef"))
+  
+  ## set all NCA metrics, except Tau to "NaN"
+  #C0 <- "NaN"; Tmax <- "NaN"; Cmax <- "NaN"; Cmax_D <- "NaN"; Tlast <- "NaN"; Clast <- "NaN"; AUClast <- "NaN"; AUMClast <- "NaN"; MRTlast <- "NaN"; No_points_Lambda_z <- "NaN"
+  #AUC_pBack_Ext <- "NaN"; AUClower_upper <- "NaN"; Rsq <- "NaN"; Rsq_adjusted <- "NaN"; Corr_XY <- "NaN"; Lambda_z <- "NaN"; Lambda_z_lower <- "NaN"; Lambda_z_upper <- "NaN"; HL_Lambda_z <- "NaN"
+  #AUCINF_obs <- "NaN"; AUCINF_D_obs <- "NaN"; AUC_pExtrap_obs <- "NaN"; Vz_obs <- "NaN"; Cl_obs <- "NaN"; AUCINF_pred <- "NaN"; AUCINF_D_pred <- "NaN"; AUC_pExtrap_pred <- "NaN"; Vz_pred <- "NaN"; Cl_pred <- "NaN"
+  #AUMCINF_obs <- "NaN"; AUMC_pExtrap_obs <- "NaN"; AUMCINF_pred <- "NaN"; AUMC_pExtrap_pred <- "NaN"; MRTINF_obs <- "NaN"; MRTINF_pred <- "NaN"; Tmin <- "NaN"; Cmin <- "NaN"; Cavg <- "NaN"
+  #AUCtau <- "NaN"; AUMCtau <- "NaN"; Clss <- "NaN"; Vss_obs <- "NaN"; Vss_pred <- "NaN"; p_Fluctuation <- "NaN"; Accumulation_Index <- "NaN"
+  
+  Tau <- ifelse(doseType=="ns", NA, Tau)
+  
+  ntime <- as.numeric(time)
+  nconc <- as.numeric(conc)
   
   # Exclude zero or negative concentration from calculations
   if (negConcExcl == "TRUE"){
@@ -150,50 +168,46 @@ est.nca <- function(time,conc,backExtrp="FALSE",negConcExcl="FALSE",doseType="ns
     if (length(zid) > 0){ntime  <- ntime[-zid]; nconc  <- nconc[-zid]}
   }
   
-  if (length(nconc) < 2){
-    C0="NaN"; Tmax <- "NaN"; Cmax <- "NaN"; Cmax_D <- "NaN"; Tlast <- "NaN"; Clast <- "NaN"; AUClast <- "NaN"; AUMClast <- "NaN"; MRTlast <- "NaN"; No_points_Lambda_z <- "NaN"
-    AUC_pBack_Ext <- "NaN"; AUClower_upper <- "NaN"; Rsq <- "NaN"; Rsq_adjusted <- "NaN"; Corr_XY <- "NaN"; Lambda_z <- "NaN"; Lambda_z_lower <- "NaN"; Lambda_z_upper <- "NaN"; HL_Lambda_z <- "NaN"
-    AUCINF_obs <- "NaN"; AUCINF_D_obs <- "NaN"; AUC_pExtrap_obs <- "NaN"; Vz_obs <- "NaN"; Cl_obs <- "NaN"; AUCINF_pred <- "NaN"; AUCINF_D_pred <- "NaN"; AUC_pExtrap_pred <- "NaN"; Vz_pred <- "NaN"; Cl_pred <- "NaN"
-    AUMCINF_obs <- "NaN"; AUMC_pExtrap_obs <- "NaN"; AUMCINF_pred <- "NaN"; AUMC_pExtrap_pred <- "NaN"; MRTINF_obs <- "NaN"; MRTINF_pred <- "NaN"; Vss_obs <- "NaN"; Vss_pred <- "NaN"
-    Tau <- "NaN"; Tmin <- "NaN"; Cmin <- "NaN"; Cavg <- "NaN"; p_Fluctuation <- "NaN"; Accumulation_Index <- "NaN"; Clss <- "NaN"
-  }else{
-    # AUClast & AUMClast and parameters for steady state data
-    AUClast <- 0.; AUMClast <- 0.; AUCnoC0 <- 0; AUClower_upper <- 0; AUCtau <- "NaN"; AUMCtau <- "NaN"
-    if (doseType == "ss"){
-      sconc <- nconc[which(ntime>=0 & ntime<Tau)]; stime <- ntime[which(ntime>=0 & ntime<Tau)]; ssnPt <- length(stime)
-      if (ssnPt != 0){
-        mIdx <- which(sconc == min(sconc[sconc!=0]))[1]; Tmin <- stime[mIdx]; Cmin <- sconc[mIdx]
-        AUCtau <- 0.; AUMCtau <- 0.
-      }else{Tmin <- "NaN"; Cmin <- "NaN"; AUCtau <- "NaN"; AUMCtau <- "NaN"}
-    }else{Tmin <- "NaN"; Cmin <- "NaN"; AUCtau <- "NaN"; AUMCtau <- "NaN"}
-    
+  if (length(nconc) >= 2){
     # Calculation of C0
     if (backExtrp == "TRUE"){
+      ndose <- ifelse(!is.null(dose), length(dose), 0)
       if (ndose == 1 | (!is.null(doseNm) & doseNumber == dose[1])){
         if (ntime[1] == 0){
-          C0 <- nconc[1]
+          C0 <- as.numeric(nconc[1])
         }else{
           if (adminType == "extravascular" | adminType == "iv-infusion"){
             C0 <- 0
             nconc <- c(C0, nconc)
             if (doseType == "ss"){
-              if (ssnPt != 0){C0 <- min(nconc); nconc <- c(C0, nconc)}
+              if (ssnPt != 0){C0 <- as.numeric(min(nconc)); nconc <- c(C0, nconc)}
             }
           }else if (adminType == "iv-bolus"){
             slope <- (nconc[2]-nconc[1])/(ntime[2]-ntime[1])
             if (nconc[1]==0 | nconc[2]==0){
-              C0 <- nconc[nconc>0][1]; nconc <- c(C0, nconc)
+              C0 <- as.numeric(nconc[nconc>0][1]); nconc <- c(C0, nconc)
             }else if (slope >= 0){
-              C0 <- nconc[1]; nconc <- c(C0, nconc)
+              C0 <- as.numeric(nconc[1]); nconc <- c(C0, nconc)
             }else{
-              C0 <- exp((ntime[1]*log(nconc[2])-ntime[2]*log(nconc[1]))/(ntime[1]-ntime[2]))
+              C0 <- as.numeric(exp((ntime[1]*log(nconc[2])-ntime[2]*log(nconc[1]))/(ntime[1]-ntime[2])))
               nconc <- c(C0, nconc) # extrapolation via log-linear regression
             }
           }
           ntime <- c(0, ntime)
         }
-      }else{C0 <- "NaN"}
-    }else{C0 <- "NaN"}
+      }
+    }
+    
+    # AUClast & AUMClast and parameters for steady state data
+    AUClast <- AUMClast <- AUCnoC0 <- AUClower_upper <- 0.
+    
+    if (doseType == "ss"){
+      sconc <- nconc[which(ntime>=0 & ntime<=Tau)]; stime <- ntime[which(ntime>=0 & ntime<=Tau)]; ssnPt <- length(stime)
+      if (ssnPt != 0){
+        mIdx <- which(sconc == min(sconc[sconc!=0]))[1]; Tmin <- stime[mIdx]; Cmin <- sconc[mIdx]
+        AUCtau <- AUMCtau <- 0.
+      }
+    }
     
     nPt    <- length(nconc)   # No. of data points
     mxId   <- which(nconc == max(nconc))[1]
@@ -211,8 +225,8 @@ est.nca <- function(time,conc,backExtrp="FALSE",negConcExcl="FALSE",doseType="ns
         AUMClast <- sum(AUMClast, delaumc)
         if (r>1 & backExtrp == "TRUE"){AUCnoC0 <- sum(AUCnoC0, delauc)}
         if (!is.null(AUCTimeRange)){if (ntime[r] >= sort(AUCTimeRange)[1] & ntime[r+1] <= sort(AUCTimeRange)[2]){AUClower_upper <- sum(AUClower_upper, delauc)}}
-        if (doseType == "ss"){
-          if (ntime[r] >= 0 & ntime[r+1] <= Tau & ssnPt != 0){AUCtau <- sum(AUCtau, delauc); AUMCtau <- sum(AUMCtau, delaumc)}
+        if (doseType == "ss" && (ntime[r] >= 0 & ntime[r+1] <= Tau & ssnPt != 0)){
+          AUCtau <- sum(AUCtau, delauc); AUMCtau <- sum(AUMCtau, delaumc)
         }
       }else if (method == "loglinear"){
         delauc   <- (nconc[r+1]-nconc[r])*(ntime[r+1]-ntime[r])/log(nconc[r+1]/nconc[r])
@@ -221,8 +235,8 @@ est.nca <- function(time,conc,backExtrp="FALSE",negConcExcl="FALSE",doseType="ns
         AUMClast <- sum(AUMClast, delaumc)
         if (r>1 & backExtrp == "TRUE"){AUCnoC0  <- sum(AUCnoC0, delauc)}
         if (!is.null(AUCTimeRange)){if (ntime[r] >= sort(AUCTimeRange)[1] & ntime[r+1] <= sort(AUCTimeRange)[2]){AUClower_upper <- sum(AUClower_upper, delauc)}}
-        if (doseType == "ss"){
-          if (ntime[r] >= 0 & ntime[r+1] <= Tau & ssnPt != 0){AUCtau <- sum(AUCtau, delauc); AUMCtau <- sum(AUMCtau, delaumc)}
+        if (doseType == "ss" && (ntime[r] >= 0 & ntime[r+1] <= Tau & ssnPt != 0)){
+          AUCtau <- sum(AUCtau, delauc); AUMCtau <- sum(AUMCtau, delaumc)
         }
       }else if (method == "mixed"){
         if (nconc[r+1]>=nconc[r] | nconc[r+1]<=0 | nconc[r]<=0){
@@ -242,27 +256,33 @@ est.nca <- function(time,conc,backExtrp="FALSE",negConcExcl="FALSE",doseType="ns
           AUMClast <- sum(AUMClast, delaumc)
           if (r>1 & backExtrp == "TRUE"){AUCnoC0  <- sum(AUCnoC0, delauc)}
           if (!is.null(AUCTimeRange)){if (ntime[r] >= sort(AUCTimeRange)[1] & ntime[r+1] <= sort(AUCTimeRange)[2]){AUClower_upper <- sum(AUClower_upper, delauc)}}
-          if (doseType == "ss"){
-            if (ntime[r] >= 0 & ntime[r+1] <= Tau & ssnPt != 0){AUCtau <- sum(AUCtau, delauc); AUMCtau <- sum(AUMCtau, delaumc)}
+          if (doseType == "ss" && (ntime[r] >= 0 & ntime[r+1] <= Tau & ssnPt != 0)){
+            AUCtau <- sum(AUCtau, delauc); AUMCtau <- sum(AUMCtau, delaumc)
           }
         }
       }
     }
     if (is.null(AUCTimeRange)){AUClower_upper <- AUClast}
-    if (backExtrp == "TRUE"){AUC_pBack_Ext <- 100*(AUClast-AUCnoC0)/AUClast}else{AUC_pBack_Ext <- "NaN"}
+    if (backExtrp == "TRUE"){AUC_pBack_Ext <- 100*(AUClast-AUCnoC0)/AUClast} #else{AUC_pBack_Ext <- "NaN"}
     
     # MRTlast
-    if (adminType != "iv-infusion"){MRTlast <- AUMClast/AUClast}else{MRTlast <- (AUMClast/AUClast)-(TI/2)}
+    if (adminType != "iv-infusion"){
+      MRTlast <- ifelse(doseType != "ss", AUMClast/AUClast, AUMCtau/AUCtau)
+    }else{
+      MRTlast <- (AUMClast/AUClast)-(TI/2)
+    }
     
-    # PK measures involving terminal-phase elimination
+    # NCA metrics involving terminal-phase elimination
     # Determine the lower and upper indeces of the time vector used for Lambda calculation
+    AUCINF_obs <- AUCINF_pred <- AUMCINF_obs <- AUMCINF_pred <- 0.
+    
     llower <- tail(which(nconc == max(nconc)), 1); lupper <- nPt
     if (!is.null(LambdaTimeRange)){
       Lambda_z_lower <- sort(LambdaTimeRange)[1]; Lambda_z_upper <- sort(LambdaTimeRange)[2]
       if (length(ntime[ntime >= Lambda_z_lower]) != 0 & length(ntime[ntime <= Lambda_z_upper]) != 0){
         llower <- head(match(ntime[ntime >= Lambda_z_lower]), 1); lupper <- tail(match(ntime[ntime <= Lambda_z_upper]), 1)
       }
-    }else{Lambda_z_lower <- "NaN"; Lambda_z_upper <- "NaN"}
+    } #else{Lambda_z_lower <- "NaN"; Lambda_z_upper <- "NaN"}
     
     lconc <- nconc[llower:lupper]; ltime <- ntime[llower:lupper]
     # exclude time points as mentioned by the user
@@ -271,12 +291,7 @@ est.nca <- function(time,conc,backExtrp="FALSE",negConcExcl="FALSE",doseType="ns
     zid <- which(lconc <= 0); if (length(zid) > 0){lconc <- lconc[-zid]; ltime <- ltime[-zid]}
     lconc <- log(lconc)
     lnPt  <- length(lconc)        # No. of data points for the descending part
-    if (lnPt < 3){
-      No_points_Lambda_z <- "NaN"; AUC_pBack_Ext <- "NaN"; AUClower_upper <- "NaN"; Rsq <- "NaN"; Rsq_adjusted <- "NaN"; Corr_XY <- "NaN"; Lambda_z <- "NaN"; Lambda_z_lower <- "NaN"; Lambda_z_upper <- "NaN"; HL_Lambda_z <- "NaN"
-      AUCINF_obs <- "NaN"; AUCINF_D_obs <- "NaN"; AUC_pExtrap_obs <- "NaN"; Vz_obs <- "NaN"; Cl_obs <- "NaN"; AUCINF_pred <- "NaN"; AUCINF_D_pred <- "NaN"; AUC_pExtrap_pred <- "NaN"; Vz_pred <- "NaN"; Cl_pred <- "NaN"
-      AUMCINF_obs <- "NaN"; AUMC_pExtrap_obs <- "NaN"; AUMCINF_pred <- "NaN"; AUMC_pExtrap_pred <- "NaN"; MRTINF_obs <- "NaN"; MRTINF_pred <- "NaN"; Vss_obs <- "NaN"; Vss_pred <- "NaN"
-      Tau <- "NaN"; Tmin <- "NaN"; Cmin <- "NaN"; Cavg <- "NaN"; p_Fluctuation <- "NaN"; Accumulation_Index <- "NaN"; Clss <- "NaN"
-    }else{
+    if (lnPt >= 3){
       # Determine the log-linear regression coefficients to calculate Lambda
       infd <- data.frame(np=numeric(0),rsq=numeric(0),arsq=numeric(0),m=numeric(0),inpt=numeric(0))
       for (r in 1:(lnPt-2)){
@@ -289,12 +304,7 @@ est.nca <- function(time,conc,backExtrp="FALSE",negConcExcl="FALSE",doseType="ns
         tmp  <- cbind(np=n,rsq=rsq,arsq=trsq,m=(coef(mlr)[2]),inpt=(coef(mlr)[1]))
         infd <- rbind(infd,tmp)
       }
-      if (nrow(infd) == 0){
-        No_points_Lambda_z <- "NaN"; AUC_pBack_Ext <- "NaN"; AUClower_upper <- "NaN"; Rsq <- "NaN"; Rsq_adjusted <- "NaN"; Corr_XY <- "NaN"; Lambda_z <- "NaN"; Lambda_z_lower <- "NaN"; Lambda_z_upper <- "NaN"; HL_Lambda_z <- "NaN"
-        AUCINF_obs <- "NaN"; AUCINF_D_obs <- "NaN"; AUC_pExtrap_obs <- "NaN"; Vz_obs <- "NaN"; Cl_obs <- "NaN"; AUCINF_pred <- "NaN"; AUCINF_D_pred <- "NaN"; AUC_pExtrap_pred <- "NaN"; Vz_pred <- "NaN"; Cl_pred <- "NaN"
-        AUMCINF_obs <- "NaN"; AUMC_pExtrap_obs <- "NaN"; AUMCINF_pred <- "NaN"; AUMC_pExtrap_pred <- "NaN"; MRTINF_obs <- "NaN"; MRTINF_pred <- "NaN"; Vss_obs <- "NaN"; Vss_pred <- "NaN"
-        Tau <- "NaN"; Tmin <- "NaN"; Cmin <- "NaN"; Cavg <- "NaN"; p_Fluctuation <- "NaN"; Accumulation_Index <- "NaN"; Clss <- "NaN"
-      }else{
+      if (nrow(infd) != 0){
         infd <- infd[order(infd$arsq,decreasing=T),]
         Rsq  <- infd$rsq[1]; Rsq_adjusted <- infd$arsq[1]; No_points_Lambda_z <- infd$np[1]; slope <- infd$m[1]; intercept <- infd$inpt[1]
         if (nrow(infd) > 1){
@@ -312,62 +322,99 @@ est.nca <- function(time,conc,backExtrp="FALSE",negConcExcl="FALSE",doseType="ns
         AUMCINF_obs  <- (exp(lconc[lnPt])/(Lambda_z**2))+(ltime[lnPt]*exp(lconc[lnPt])/Lambda_z)
         lastPt       <- exp((slope*ltime[lnPt])+intercept)
         AUCINF_pred  <- lastPt/Lambda_z
-        AUMCINF_pred <- (lastPt/(Lambda_z**2))+(ltime[lnPt]*lastPt/Lambda_z)
+        AUMCINF_pred <- (lastPt/(Lambda_z**2))+(ltime[lnPt]*lastPt/Lambda_z)        
       }
       
-      if (AUClast != "NaN" & AUCINF_obs != "NaN"){
+      if (AUClast != 0 & AUCINF_obs != 0){
         AUCINF_obs  <- AUClast+AUCINF_obs;  AUCINF_D_obs  <- AUCINF_obs/doseAmt;  AUC_pExtrap_obs  <- 100*(AUCINF_obs-AUClast)/AUCINF_obs
         AUCINF_pred <- AUClast+AUCINF_pred; AUCINF_D_pred <- AUCINF_pred/doseAmt; AUC_pExtrap_pred <- 100*(AUCINF_pred-AUClast)/AUCINF_pred
         Vz_obs  <- doseAmt/(Lambda_z*AUCINF_obs);  Cl_obs  <- doseAmt/AUCINF_obs
         Vz_pred <- doseAmt/(Lambda_z*AUCINF_pred); Cl_pred <- doseAmt/AUCINF_pred
-      }else{
-        AUCINF_obs <- "NaN"; AUCINF_D_obs <- "NaN"; AUC_pExtrap_obs <- "NaN"; AUCINF_pred <- "NaN"; AUCINF_D_pred <- "NaN"; AUC_pExtrap_pred <- "NaN"; Vz_obs <- "NaN"; Vz_pred <- "NaN"; Cl_obs <- "NaN"; Cl_pred <- "NaN"
       }
       
-      if(AUMClast != "NaN" & AUMCINF_obs != "NaN"){
+      if(AUMClast != 0 & AUMCINF_obs != 0){
         AUMCINF_obs  <- AUMClast+AUMCINF_obs;  AUMCINF_D_obs  <- AUMCINF_obs/doseAmt;  AUMC_pExtrap_obs  <- 100*(AUMCINF_obs-AUMClast)/AUMCINF_obs
         AUMCINF_pred <- AUMClast+AUMCINF_pred; AUMCINF_D_pred <- AUMCINF_pred/doseAmt; AUMC_pExtrap_pred <- 100*(AUMCINF_pred-AUMClast)/AUMCINF_pred
-      }else{
-        AUMCINF_obs <- "NaN"; AUMCINF_D_obs <- "NaN"; AUMC_pExtrap_obs <- "NaN"; AUMCINF_pred <- "NaN"; AUMCINF_D_pred <- "NaN"; AUMC_pExtrap_pred <- "NaN"
       }
       
-      if (AUCINF_obs  != "NaN" & AUMCINF_obs  != "NaN"){if (adminType != "iv-infusion"){MRTINF_obs  <- AUMCINF_obs/AUCINF_obs}else{MRTINF_obs  <- (AUMCINF_obs/AUCINF_obs)-(TI/2)}}else{MRTINF_obs  <- "NaN"}
-      if (AUCINF_pred != "NaN" & AUMCINF_pred != "NaN"){if (adminType != "iv-infusion"){MRTINF_pred <- AUMCINF_pred/AUCINF_pred}else{MRTINF_pred <- (AUMCINF_pred/AUCINF_pred)-(TI/2)}}else{MRTINF_pred <- "NaN"}
-      if (MRTINF_obs  != "NaN" & doseType != "ss"){Vss_obs  <- MRTINF_obs*Cl_obs}else{Vss_obs  ="NaN"}
-      if (MRTINF_pred != "NaN" & doseType != "ss"){Vss_pred <- MRTINF_pred*Cl_pred}else{Vss_pred ="NaN"}
+      if (AUCINF_obs  != 0 & AUMCINF_obs  != 0){MRTINF_obs  <- ifelse(adminType != "iv-infusion", AUMCINF_obs/AUCINF_obs,   ((AUMCINF_obs/AUCINF_obs)-(TI/2)))} #else{MRTINF_obs  <- "NaN"}
+      if (AUCINF_pred != 0 & AUMCINF_pred != 0){MRTINF_pred <- ifelse(adminType != "iv-infusion", AUMCINF_pred/AUCINF_pred, ((AUMCINF_pred/AUCINF_pred)-(TI/2)))} #else{MRTINF_pred <- "NaN"}
       
-      if (doseType == "ss" & AUCtau != 0){
-        if (ssnPt != 0){
-          Cavg <- AUCtau/Tau
-          p_Fluctuation <- 100*(Cmax-Cmin)/Cavg
-          if (Lambda_z != "NaN"){Accumulation_Index <- 1/(1-exp(-Lambda_z*Tau))}else{Accumulation_Index <- "NaN"}
-          Clss <- doseAmt/AUCtau
-          Cmax <- max(sconc)
+      if (doseType == "ss" && ssnPt != 0){
+        Cavg <- AUCtau/Tau
+        Clss <- doseAmt/AUCtau
+        Cmax <- max(sconc)
+        p_Fluctuation <- 100*(Cmax-Cmin)/Cavg
+        if (Lambda_z != "NaN"){Accumulation_Index <- 1/(1-exp(-Lambda_z*Tau))}
+        
+        # re-define MRTINF for steady state and calculate Vss
+        if (AUCINF_obs  != "NaN" & AUMCINF_obs  != "NaN" & AUCtau != 0){
           if (adminType == "iv-infusion"){
             MRTINF_obs <- ((AUMCtau+Tau*(AUCINF_obs-AUCtau))/(AUCtau)-(TI/2)); MRTINF_pred <- ((AUMCtau+Tau*(AUCINF_pred-AUCtau))/(AUCtau)-(TI/2))
           }else{
             MRTINF_obs <- (AUMCtau+Tau*(AUCINF_obs-AUCtau))/AUCtau; MRTINF_pred <- (AUMCtau+Tau*(AUCINF_pred-AUCtau))/AUCtau
           }
-          if (Lambda_z != "NaN"){Vz <- doseAmt/(Lambda_z*AUCtau)}else{Vz <- "NaN"}
-          if (adminType != "extravascular"){Vss_obs <- MRTINF_obs*Clss; Vss_pred <- MRTINF_pred*Clss}
+          Vss_obs  <- MRTINF_obs*Clss
+          Vss_pred <- MRTINF_pred*Clss
         }
-      }else if (doseType == "ss" & AUCtau == 0){
-        if (ssnPt != 0){
-          if (Lambda_z != "NaN"){Accumulation_Index <- 1/(1-exp(-Lambda_z*Tau))}else{Accumulation_Index <- "NaN"}
-          Cavg <- "NaN"; p_Fluctuation <- "NaN"; Clss <- "NaN"; Cmax <- "NaN"; MRTINF_obs <- "NaN"; MRTINF_pred <- "NaN"; Vz <- "NaN"; Vss_obs <- "NaN"; Vss_pred <- "NaN"
-        }
-      }else if (doseType == "ss"){
-        if (ssnPt == 0){Cavg <- "NaN"; p_Fluctuation <- "NaN"; Accumulation_Index <- "NaN"; Clss <- "NaN"; Cmax <- "NaN"; MRTINF_obs <- "NaN"; MRTINF_pred <- "NaN"; Vz <- "NaN"; Vss_obs <- "NaN"; Vss_pred <- "NaN"}
-      }else if (doseType != "ss"){Cavg <- "NaN"; p_Fluctuation <- "NaN"; Accumulation_Index <- "NaN"; Clss <- "NaN"; Vz <- "NaN"; Vss_obs <- "NaN"; Vss_pred <- "NaN"}
+      }
     }
   }
   
-  if (doseType != "ss"){Tau <- "NaN"}
-  if (adminType != "iv-infusion"){TI <- "NaN"}
+  if(!exists("C0")) C0 <- NA
+  if(!exists("Tmax")) Tmax <- NA
+  if(!exists("Cmax")) Cmax <- NA
+  if(!exists("Cmax_D")) Cmax_D <- NA
+  if(!exists("Tlast")) Tlast <- NA
+  if(!exists("Clast")) Clast <- NA
+  if(!exists("AUClast")) AUClast <- NA
+  if(!exists("AUMClast")) AUMClast <- NA
+  if(!exists("MRTlast")) MRTlast <- NA
+  if(!exists("No_points_Lambda_z")) No_points_Lambda_z <- NA
+  if(!exists("AUC_pBack_Ext")) AUC_pBack_Ext <- NA
+  if(!exists("AUClower_upper")) AUClower_upper <- NA
+  if(!exists("Rsq")) Rsq <- NA
+  if(!exists("Rsq_adjusted")) Rsq_adjusted <- NA
+  if(!exists("Corr_XY")) Corr_XY <- NA
+  if(!exists("Lambda_z")) Lambda_z <- NA
+  if(!exists("Lambda_z_lower")) Lambda_z_lower <- NA
+  if(!exists("Lambda_z_upper")) Lambda_z_upper <- NA
+  if(!exists("HL_Lambda_z")) HL_Lambda_z <- NA
+  if(!exists("AUCINF_obs")) AUCINF_obs <- NA
+  if(!exists("AUCINF_D_obs")) AUCINF_D_obs <- NA
+  if(!exists("AUC_pExtrap_obs")) AUC_pExtrap_obs <- NA
+  if(!exists("Vz_obs")) Vz_obs <- NA
+  if(!exists("Cl_obs")) Cl_obs <- NA
+  if(!exists("AUCINF_pred")) AUCINF_pred <- NA
+  if(!exists("AUCINF_D_pred")) AUCINF_D_pred <- NA
+  if(!exists("AUC_pExtrap_pred")) AUC_pExtrap_pred <- NA
+  if(!exists("Vz_pred")) Vz_pred <- NA
+  if(!exists("Cl_pred")) Cl_pred <- NA
+  if(!exists("AUMCINF_obs")) AUMCINF_obs <- NA
+  if(!exists("AUMC_pExtrap_obs")) AUMC_pExtrap_obs <- NA
+  if(!exists("AUMCINF_pred")) AUMCINF_pred <- NA
+  if(!exists("AUMC_pExtrap_pred")) AUMC_pExtrap_pred <- NA
+  if(!exists("MRTINF_obs")) MRTINF_obs <- NA
+  if(!exists("MRTINF_pred")) MRTINF_pred <- NA
+  if(!exists("Tmin")) Tmin <- NA
+  if(!exists("Cmin")) Cmin <- NA
+  if(!exists("Cavg")) Cavg <- NA
+  if(!exists("AUCtau")) AUCtau <- NA
+  if(!exists("AUMCtau")) AUMCtau <- NA
+  if(!exists("Clss")) Clss <- NA
+  if(!exists("Vss_obs")) Vss_obs <- NA
+  if(!exists("Vss_pred")) Vss_pred <- NA
+  if(!exists("p_Fluctuation")) p_Fluctuation <- NA
+  if(!exists("Accumulation_Index")) Accumulation_Index <- NA
+  
   if (!is.null(simFile) & dset == "obs"){
-    NCAprm <- c(C0,Tmax,0,0,0,Cmax,0,0,0,Cmax_D,Tlast,Clast,AUClast,0,0,0,AUMClast,0,0,0,MRTlast,No_points_Lambda_z,AUC_pBack_Ext,AUClower_upper,0,0,0,Rsq,Rsq_adjusted,Corr_XY,Lambda_z,Lambda_z_lower,Lambda_z_upper,HL_Lambda_z,0,0,0,AUCINF_obs,0,0,0,AUCINF_D_obs,AUC_pExtrap_obs,Vz_obs,Cl_obs,AUCINF_pred,AUCINF_D_pred,AUC_pExtrap_pred,Vz_pred,Cl_pred,AUMCINF_obs,0,0,0,AUMC_pExtrap_obs,AUMCINF_pred,AUMC_pExtrap_pred,MRTINF_obs,MRTINF_pred,Vss_obs,Vss_pred,Tau,Tmin,Cmin,Cavg,p_Fluctuation,Accumulation_Index,Clss)
+    NCAprm <- as.numeric(c(C0,Tmax,0,0,0,Cmax,0,0,0,Cmax_D,Tlast,Clast,AUClast,0,0,0,AUMClast,0,0,0,MRTlast,No_points_Lambda_z,AUC_pBack_Ext,AUClower_upper,0,0,0,Rsq,Rsq_adjusted,Corr_XY,Lambda_z,Lambda_z_lower,Lambda_z_upper,HL_Lambda_z,0,0,0,AUCINF_obs,0,0,0,AUCINF_D_obs,AUC_pExtrap_obs,Vz_obs,Cl_obs,AUCINF_pred,0,0,0,AUCINF_D_pred,AUC_pExtrap_pred,Vz_pred,Cl_pred,AUMCINF_obs,AUMC_pExtrap_obs,AUMCINF_pred,AUMC_pExtrap_pred,MRTINF_obs,MRTINF_pred,Tau,Tmin,Cmin,Cavg,AUCtau,AUMCtau,Clss,Vss_obs,Vss_pred,p_Fluctuation,Accumulation_Index))
+    
+    names(NCAprm) <- c("C0","Tmax","simTmax","dTmax","npdeTmax","Cmax","simCmax","dCmax","npdeCmax","Cmax_D","Tlast","Clast","AUClast","simAUClast","dAUClast","npdeAUClast","AUMClast","simAUMClast","dAUMClast","npdeAUMClast","MRTlast","No_points_Lambda_z","AUC_pBack_Ext","AUClower_upper","simAUClower_upper","dAUClower_upper","npdeAUClower_upper","Rsq","Rsq_adjusted","Corr_XY","Lambda_z","Lambda_z_lower","Lambda_z_upper","HL_Lambda_z","simHL_Lambda_z","dHL_Lambda_z","npdeHL_Lambda_z","AUCINF_obs","simAUCINF_obs","dAUCINF_obs","npdeAUCINF_obs","AUCINF_D_obs","AUC_pExtrap_obs","Vz_obs","Cl_obs","AUCINF_pred","simAUCINF_pred","dAUCINF_pred","npdeAUCINF_pred","AUCINF_D_pred","AUC_pExtrap_pred","Vz_pred","Cl_pred","AUMCINF_obs","AUMC_pExtrap_obs","AUMCINF_pred","AUMC_pExtrap_pred","MRTINF_obs","MRTINF_pred","Tau","Tmin","Cmin","Cavg","AUCtau","AUMCtau","Clss","Vss_obs","Vss_pred","p_Fluctuation","Accumulation_Index")
   }else{
-    NCAprm <- c(C0,Tmax,Cmax,Cmax_D,Tlast,Clast,AUClast,AUMClast,MRTlast,No_points_Lambda_z,AUC_pBack_Ext,AUClower_upper,Rsq,Rsq_adjusted,Corr_XY,Lambda_z,Lambda_z_lower,Lambda_z_upper,HL_Lambda_z,AUCINF_obs,AUCINF_D_obs,AUC_pExtrap_obs,Vz_obs,Cl_obs,AUCINF_pred,AUCINF_D_pred,AUC_pExtrap_pred,Vz_pred,Cl_pred,AUMCINF_obs,AUMC_pExtrap_obs,AUMCINF_pred,AUMC_pExtrap_pred,MRTINF_obs,MRTINF_pred,Vss_obs,Vss_pred,Tau,Tmin,Cmin,Cavg,p_Fluctuation,Accumulation_Index,Clss)
+    NCAprm <- as.numeric(c(C0,Tmax,Cmax,Cmax_D,Tlast,Clast,AUClast,AUMClast,MRTlast,No_points_Lambda_z,AUC_pBack_Ext,AUClower_upper,Rsq,Rsq_adjusted,Corr_XY,Lambda_z,Lambda_z_lower,Lambda_z_upper,HL_Lambda_z,AUCINF_obs,AUCINF_D_obs,AUC_pExtrap_obs,Vz_obs,Cl_obs,AUCINF_pred,AUCINF_D_pred,AUC_pExtrap_pred,Vz_pred,Cl_pred,AUMCINF_obs,AUMC_pExtrap_obs,AUMCINF_pred,AUMC_pExtrap_pred,MRTINF_obs,MRTINF_pred,Tau,Tmin,Cmin,Cavg,AUCtau,AUMCtau,Clss,Vss_obs,Vss_pred,p_Fluctuation,Accumulation_Index))
+    
+    names(NCAprm) <- c("C0","Tmax","Cmax","Cmax_D","Tlast","Clast","AUClast","AUMClast","MRTlast","No_points_Lambda_z","AUC_pBack_Ext","AUClower_upper","Rsq","Rsq_adjusted","Corr_XY","Lambda_z","Lambda_z_lower","Lambda_z_upper","HL_Lambda_z","AUCINF_obs","AUCINF_D_obs","AUC_pExtrap_obs","Vz_obs","Cl_obs","AUCINF_pred","AUCINF_D_pred","AUC_pExtrap_pred","Vz_pred","Cl_pred","AUMCINF_obs","AUMC_pExtrap_obs","AUMCINF_pred","AUMC_pExtrap_pred","MRTINF_obs","MRTINF_pred","Tau","Tmin","Cmin","Cavg","AUCtau","AUMCtau","Clss","Vss_obs","Vss_pred","p_Fluctuation","Accumulation_Index")
   }
   return(NCAprm)
 }
